@@ -66,6 +66,7 @@ struct key_tok ntp_keywords[] = {
 { "discard",		T_Discard,		FOLLBY_TOKEN },
 { "reset",		T_Reset,		FOLLBY_TOKEN },
 { "restrict",		T_Restrict,		FOLLBY_TOKEN },
+{ "rlimit",		T_Rlimit,		FOLLBY_TOKEN },
 { "server",		T_Server,		FOLLBY_STRING },
 { "setvar",		T_Setvar,		FOLLBY_STRING },
 { "statistics",		T_Statistics,		FOLLBY_TOKEN },
@@ -96,6 +97,7 @@ struct key_tok ntp_keywords[] = {
 { "iburst",		T_Iburst,		FOLLBY_TOKEN },
 { "key",		T_Key,			FOLLBY_TOKEN },
 { "maxpoll",		T_Maxpoll,		FOLLBY_TOKEN },
+{ "mdnstries",		T_Mdnstries,		FOLLBY_TOKEN },
 { "minpoll",		T_Minpoll,		FOLLBY_TOKEN },
 { "mode",		T_Mode,			FOLLBY_TOKEN },
 { "noselect",		T_Noselect,		FOLLBY_TOKEN },
@@ -198,8 +200,14 @@ struct key_tok ntp_keywords[] = {
 { "ntp",		T_Ntp,			FOLLBY_TOKEN },
 { "mode7",		T_Mode7,		FOLLBY_TOKEN },
 { "stats",		T_Stats,		FOLLBY_TOKEN },
+/* rlimit_option */
+{ "memlock",		T_Memlock,		FOLLBY_TOKEN },
+{ "stacksize",		T_Stacksize,		FOLLBY_TOKEN },
+{ "filenum",		T_Filenum,		FOLLBY_TOKEN },
 /* tinker_option */
 { "step",		T_Step,			FOLLBY_TOKEN },
+{ "stepback",		T_Stepback,		FOLLBY_TOKEN },
+{ "stepfwd",		T_Stepfwd,		FOLLBY_TOKEN },
 { "panic",		T_Panic,		FOLLBY_TOKEN },
 { "dispersion",		T_Dispersion,		FOLLBY_TOKEN },
 { "stepout",		T_Stepout,		FOLLBY_TOKEN },
@@ -209,7 +217,6 @@ struct key_tok ntp_keywords[] = {
 /* miscellaneous_command */
 { "port",		T_Port,			FOLLBY_TOKEN },
 { "interface",		T_Interface,		FOLLBY_TOKEN },
-{ "qos",		T_Qos,			FOLLBY_TOKEN },
 { "saveconfigdir",	T_Saveconfigdir,	FOLLBY_STRING },
 /* interface_command (ignore and interface already defined) */
 { "nic",		T_Nic,			FOLLBY_TOKEN },
@@ -326,7 +333,6 @@ generate_fsm(void)
 	char *r;
 	u_short initial_state;
 	u_short this_state;
-	u_short prev_state;
 	u_short state;
 	u_short i;
 	u_short token;
@@ -353,7 +359,7 @@ generate_fsm(void)
 			fprintf(stderr,
 				"keyword-gen sst[%u] too small "
 				"for keyword '%s' id %d\n",
-				COUNTOF(sst),
+				(int)COUNTOF(sst),
 				ntp_keywords[i].key,
 				token);
 			exit(4);
@@ -435,7 +441,6 @@ generate_fsm(void)
 		 * spellings result in the same T_* value.
 		 */
 			prefix_len = 0;
-			prev_state = 0;
 			this_state = i;
 			do {
 				for (state = 1; state < sst_highwater; state++)
