@@ -375,7 +375,8 @@ leapsec_reset_frame(void)
 int/*BOOL*/
 leapsec_load_stream(
 	FILE       * ifp  ,
-	const char * fname)
+	const char * fname,
+	int/*BOOL*/  logall)
 {
 	leap_table_t *pt;
 	int           rcheck;
@@ -384,30 +385,31 @@ leapsec_load_stream(
 		fname = "<unknown>";
 
 	rcheck = leapsec_validate((leapsec_reader)getc, ifp);
-	switch (rcheck)
-	{
-	case LSVALID_GOODHASH:
-		msyslog(LOG_NOTICE, "%s ('%s'): good hash signature",
-			logPrefix, fname);
-		break;
-
-	case LSVALID_NOHASH:
-		msyslog(LOG_NOTICE, "%s ('%s'): no hash signature",
-			logPrefix, fname);
-		break;
-	case LSVALID_BADHASH:
-		msyslog(LOG_ERR, "%s ('%s'): signature mismatch",
-			logPrefix, fname);
-		break;
-	case LSVALID_BADFORMAT:
-		msyslog(LOG_ERR, "%s ('%s'): malformed hash signature",
-			logPrefix, fname);
-		break;
-	default:
-		msyslog(LOG_ERR, "%s ('%s'): unknown error code %d",
-			logPrefix, fname, rcheck);
-		break;
-	}
+	if (logall)
+		switch (rcheck)
+		{
+		case LSVALID_GOODHASH:
+			msyslog(LOG_NOTICE, "%s ('%s'): good hash signature",
+				logPrefix, fname);
+			break;
+			
+		case LSVALID_NOHASH:
+			msyslog(LOG_ERR, "%s ('%s'): no hash signature",
+				logPrefix, fname);
+			break;
+		case LSVALID_BADHASH:
+			msyslog(LOG_ERR, "%s ('%s'): signature mismatch",
+				logPrefix, fname);
+			break;
+		case LSVALID_BADFORMAT:
+			msyslog(LOG_ERR, "%s ('%s'): malformed hash signature",
+				logPrefix, fname);
+			break;
+		default:
+			msyslog(LOG_ERR, "%s ('%s'): unknown error code %d",
+				logPrefix, fname, rcheck);
+			break;
+		}
 	if (rcheck < 0)
 		return FALSE;
 
@@ -449,7 +451,8 @@ int/*BOOL*/
 leapsec_load_file(
 	const char  * fname,
 	struct stat * sb_old,
-	int/*BOOL*/   force)
+	int/*BOOL*/   force,
+	int/*BOOL*/   logall)
 {
 	FILE       * fp;
 	struct stat  sb_new;
@@ -461,8 +464,9 @@ leapsec_load_file(
 	
 	/* try to stat the leapfile */
 	if (0 != stat(fname, &sb_new)) {
-		msyslog(LOG_ERR, "%s ('%s'): stat failed: %m",
-			logPrefix, fname);
+		if (logall)
+			msyslog(LOG_ERR, "%s ('%s'): stat failed: %m",
+				logPrefix, fname);
 		return FALSE;
 	}
 
@@ -494,13 +498,14 @@ leapsec_load_file(
 	 */
 	/* coverity[toctou] */
 	if ((fp = fopen(fname, "r")) == NULL) {
-		msyslog(LOG_ERR,
-			"%s ('%s'): open failed: %m",
-			logPrefix, fname);
+		if (logall)
+			msyslog(LOG_ERR,
+				"%s ('%s'): open failed: %m",
+				logPrefix, fname);
 		return FALSE;
 	}
 
-	rc = leapsec_load_stream(fp, fname);
+	rc = leapsec_load_stream(fp, fname, logall);
 	fclose(fp);
 	return rc;
 }
@@ -782,7 +787,7 @@ leapsec_add(
 	const vint64 * now64 ,
 	int            insert)
 {
-	vint64		ttime, stime;
+	vint64		ttime, starttime;
 	struct calendar	fts;
 	leap_info_t	li;
 
@@ -810,12 +815,12 @@ leapsec_add(
 	fts.hour     = 0;
 	fts.minute   = 0;
 	fts.second   = 0;
-	stime = ntpcal_date_to_ntp64(&fts);
+	starttime = ntpcal_date_to_ntp64(&fts);
 	fts.month++;
 	ttime = ntpcal_date_to_ntp64(&fts);
 
 	li.ttime = ttime;
-	li.stime = ttime.D_s.lo - stime.D_s.lo;
+	li.stime = ttime.D_s.lo - starttime.D_s.lo;
 	li.taiof = (pt->head.size ? pt->info[0].taiof : pt->head.base_tai)
 	         + (insert ? 1 : -1);
 	li.dynls = 1;
@@ -834,7 +839,7 @@ leapsec_raw(
 	int            taiof,
 	int            dynls)
 {
-	vint64		stime;
+	vint64		starttime;
 	struct calendar	fts;
 	leap_info_t	li;
 
@@ -851,9 +856,9 @@ leapsec_raw(
 		return FALSE;
 	}
 	fts.month--; /* was in range 1..12, no overflow here! */
-	stime    = ntpcal_date_to_ntp64(&fts);
+	starttime    = ntpcal_date_to_ntp64(&fts);
 	li.ttime = *ttime;
-	li.stime = ttime->D_s.lo - stime.D_s.lo;
+	li.stime = ttime->D_s.lo - starttime.D_s.lo;
 	li.taiof = (int16_t)taiof;
 	li.dynls = (dynls != 0);
 	return add_range(pt, &li);
@@ -915,7 +920,8 @@ do_leap_hash(
 	/* now do the byte twiddle */
 	for (wi=0; wi < 5; ++wi)
 		for (di=3; di >= 0; --di) {
-			mac->hv[wi*4 + di] = (unsigned char)tmp[wi];
+			mac->hv[wi*4 + di] =
+				(unsigned char)(tmp[wi] & 0x0FF);
 			tmp[wi] >>= 8;
 		}
 	return TRUE;
@@ -967,7 +973,7 @@ leapsec_validate(
 			do_hash_data(&mdctx, line+2);
 		else if (!strncmp(line, "#$", 2))
 			do_hash_data(&mdctx, line+2);
-		else if (isdigit(line[0]))
+		else if (isdigit((unsigned char)line[0]))
 			do_hash_data(&mdctx, line);
 	}
 	isc_sha1_final(&mdctx, ldig.hv);
