@@ -44,6 +44,7 @@ struct dns_ctx {
 	int		flags;
 #define CTX_BCST	0x0001
 #define CTX_UCST	0x0002
+#define CTX_xCST	0x0003
 #define CTX_CONC	0x0004
 #define CTX_unused	0xfffd
 	int		key_id;
@@ -69,13 +70,14 @@ struct xmt_ctx_tag {
 	sent_pkt *		spkt;
 };
 
-struct timeval	headspace;
+struct timeval	gap;
 xmt_ctx *	xmt_q;
 struct key *	keys = NULL;
-struct timeval	bcst_timeout_tv;
-int		ucst_timeout;
+int		response_timeout;
+struct timeval	response_tv;
+struct timeval	start_tv;
 /* check the timeout at least once per second */
-struct timeval	ucst_wakeup_tv = { 0, 888888 };
+struct timeval	wakeup_tv = { 0, 888888 };
 
 sent_pkt *	fam_listheads[2];
 #define v4_pkts_list	(fam_listheads[0])
@@ -83,7 +85,7 @@ sent_pkt *	fam_listheads[2];
 
 static union {
 	struct pkt pkt;
-	char   buf[1500];
+	char   buf[LEN_PKT_NOMAC + NTP_MAXEXTEN + MAX_MAC_LEN];
 } rbuf;
 
 #define r_pkt  rbuf.pkt
@@ -92,6 +94,7 @@ static union {
 int droproot;			/* intres imports these */
 int root_dropped;
 #endif
+u_long current_time;		/* libntp/authkeys.c */
 
 void open_sockets(void);
 void handle_lookup(const char *name, int flags);
@@ -112,7 +115,6 @@ void sock_cb(evutil_socket_t, short, void *);
 void check_exit_conditions(void);
 void sntp_libevent_log_cb(int, const char *);
 void set_li_vn_mode(struct pkt *spkt, char leap, char version, char mode);
-int  sntp_main(int argc, char **argv);
 int  set_time(double offset);
 void dec_pending_ntp(const char *, sockaddr_u *);
 int  libevent_version_ok(void);
@@ -125,7 +127,8 @@ int  gettimeofday_cached(struct event_base *b, struct timeval *tv);
 int
 sntp_main (
 	int argc,
-	char **argv
+	char **argv,
+	const char *sntpVersion
 	)
 {
 	int			i;
@@ -140,13 +143,14 @@ sntp_main (
 		exit(EX_SOFTWARE);
 
 	init_lib();
+	init_auth();
 
 	optct = ntpOptionProcess(&sntpOptions, argc, argv);
 	argc -= optct;
 	argv += optct;
 
-	debug = DESC(DEBUG_LEVEL).optOccCt;
-	TRACE(1, ("%s\n", Version));
+
+	debug = OPT_VALUE_SET_DEBUG_LEVEL;
 
 	TRACE(2, ("init_lib() done, %s%s\n",
 		  (ipv4_works)
@@ -157,11 +161,13 @@ sntp_main (
 		      : ""));
 	ntpver = OPT_VALUE_NTPVERSION;
 	steplimit = OPT_VALUE_STEPLIMIT / 1e3;
-	headspace.tv_usec = max(0, OPT_VALUE_HEADSPACE * 1000);
-	headspace.tv_usec = min(headspace.tv_usec, 999999);
+	gap.tv_usec = max(0, OPT_VALUE_GAP * 1000);
+	gap.tv_usec = min(gap.tv_usec, 999999);
 
-	if (HAVE_OPT(FILELOG))
-		open_logfile(OPT_ARG(FILELOG));
+	if (HAVE_OPT(LOGFILE))
+		open_logfile(OPT_ARG(LOGFILE));
+
+	msyslog(LOG_INFO, "%s", sntpVersion);
 
 	if (0 == argc && !HAVE_OPT(BROADCAST) && !HAVE_OPT(CONCURRENT)) {
 		printf("%s: Must supply at least one of -b hostname, -c hostname, or hostname.\n",
@@ -172,13 +178,13 @@ sntp_main (
 
 	/*
 	** Eventually, we probably want:
-	** - separate bcst and ucst timeouts
+	** - separate bcst and ucst timeouts (why?)
 	** - multiple --timeout values in the commandline
 	*/
-	bcst_timeout_tv.tv_sec = OPT_VALUE_BCTIMEOUT;
-	bcst_timeout_tv.tv_usec = 0;
 
-	ucst_timeout = OPT_VALUE_UCTIMEOUT;
+	response_timeout = OPT_VALUE_TIMEOUT;
+	response_tv.tv_sec = response_timeout;
+	response_tv.tv_usec = 0;
 
 	/* IPv6 available? */
 	if (isc_net_probeipv6() != ISC_R_SUCCESS) {
@@ -266,6 +272,7 @@ sntp_main (
 	for (i = 0; i < argc; ++i)
 		handle_lookup(argv[i], CTX_UCST);
 
+	gettimeofday_cached(base, &start_tv);
 	event_base_dispatch(base);
 	event_base_free(base);
 
@@ -288,9 +295,7 @@ open_sockets(
 	)
 {
 	sockaddr_u	name;
-	int		one_fam_works;
 
-	one_fam_works = FALSE;
 	if (-1 == sock4) {
 		sock4 = socket(PF_INET, SOCK_DGRAM, 0);
 		if (-1 == sock4) {
@@ -321,8 +326,7 @@ open_sockets(
 			msyslog(LOG_ERR,
 				"open_sockets: event_new(base, sock4) failed!");
 		} else {
-			one_fam_works = TRUE;
-			event_add(ev_sock4, &ucst_wakeup_tv);
+			event_add(ev_sock4, &wakeup_tv);
 		}
 	}
 
@@ -351,13 +355,12 @@ open_sockets(
 		/* Register an NTP callback for recv/timeout */
 		ev_sock6 = event_new(base, sock6,
 				     EV_TIMEOUT | EV_READ | EV_PERSIST,
-		 		     &sock_cb, NULL);
+				     &sock_cb, NULL);
 		if (NULL == ev_sock6) {
 			msyslog(LOG_ERR,
 				"open_sockets: event_new(base, sock6) failed!");
 		} else {
-			one_fam_works = TRUE;
-			event_add(ev_sock6, &ucst_wakeup_tv);
+			event_add(ev_sock6, &wakeup_tv);
 		}
 	}
 	
@@ -395,15 +398,13 @@ handle_lookup(
 	hints.ai_protocol = IPPROTO_UDP;
 
 	name_sz = 1 + strlen(name);
-	octets = sizeof(*ctx) + name_sz;
-	ctx = emalloc_zero(octets);
-	name_copy = (char *)(ctx + 1);
-	memcpy(name_copy, name, name_sz);
-	ctx->name = name_copy;
+	octets = sizeof(*ctx) + name_sz;	// Space for a ctx and the name
+	ctx = emalloc_zero(octets);		// ctx at ctx[0]
+	name_copy = (char *)(ctx + 1);		// Put the name at ctx[1]
+	memcpy(name_copy, name, name_sz);	// copy the name to ctx[1]
+	ctx->name = name_copy;			// point to it...
 	ctx->flags = flags;
-	ctx->timeout = (CTX_BCST & flags)
-			   ? bcst_timeout_tv
-			   : ucst_wakeup_tv;
+	ctx->timeout = response_tv;
 
 	/* The following should arguably be passed in... */
 	if (ENABLED_OPT(AUTHENTICATION) &&
@@ -498,11 +499,6 @@ sntp_name_resolved(
 				break;
 			}
 
-			spkt = emalloc_zero(sizeof(*spkt));
-			spkt->dctx = dctx;
-			octets = min(ai->ai_addrlen, sizeof(spkt->addr));
-			memcpy(&spkt->addr, ai->ai_addr, octets);
-
 			/*
 			** We're waiting for a response for either unicast
 			** or broadcast, so...
@@ -510,8 +506,13 @@ sntp_name_resolved(
 			++n_pending_ntp;
 
 			/* If this is for a unicast IP, queue a request */
-			if (dctx->flags & CTX_UCST)
+			if (dctx->flags & CTX_UCST) {
+				spkt = emalloc_zero(sizeof(*spkt));
+				spkt->dctx = dctx;
+				octets = min(ai->ai_addrlen, sizeof(spkt->addr));
+				memcpy(&spkt->addr, ai->ai_addr, octets);
 				queue_xmt(sock, dctx, spkt, xmt_delay);
+			}
 		}
 	}
 	/* n_pending_dns really should be >0 here... */
@@ -629,10 +630,10 @@ xmt_timer_cb(
 			return;
 	}
 	if (xmt_q->sched <= start_cb.tv_sec) {
-		event_add(ev_xmt_timer, &headspace);
-		TRACE(2, ("xmt_timer_cb: at .%6.6u headspace %6.6u\n",
+		event_add(ev_xmt_timer, &gap);
+		TRACE(2, ("xmt_timer_cb: at .%6.6u gap %6.6u\n",
 			  (u_int)start_cb.tv_usec,
-			  (u_int)headspace.tv_usec));
+			  (u_int)gap.tv_usec));
 	} else {
 		delay.tv_sec = xmt_q->sched - start_cb.tv_sec;
 		delay.tv_usec = 0;
@@ -658,7 +659,8 @@ xmt(
 	sockaddr_u *	dst = &spkt->addr;
 	struct timeval	tv_xmt;
 	struct pkt	x_pkt;
-	int		pkt_len;
+	size_t		pkt_len;
+	int		sent;
 
 	if (0 != gettimeofday(&tv_xmt, NULL)) {
 		msyslog(LOG_ERR,
@@ -670,27 +672,25 @@ xmt(
 	pkt_len = generate_pkt(&x_pkt, &tv_xmt, dctx->key_id,
 			       dctx->key);
 
-	/* The current sendpkt does not return status */
-	sendpkt(sock, dst, &x_pkt, pkt_len);
-	/* Save the packet we sent... */
-	memcpy(&spkt->x_pkt, &x_pkt, min(sizeof(spkt->x_pkt), pkt_len));
-	spkt->stime = tv_xmt.tv_sec - JAN_1970;
+	sent = sendpkt(sock, dst, &x_pkt, pkt_len);
+	if (sent) {
+		/* Save the packet we sent... */
+		memcpy(&spkt->x_pkt, &x_pkt, min(sizeof(spkt->x_pkt),
+		       pkt_len));
+		spkt->stime = tv_xmt.tv_sec - JAN_1970;
 
-	TRACE(2, ("xmt: %lx.%6.6u %s %s\n", (u_long)tv_xmt.tv_sec,
-		  (u_int)tv_xmt.tv_usec, dctx->name, stoa(dst)));
-
-	/*
-	** If the send fails:
-	** - decrement n_pending_ntp
-	** - restart the loop
-	*/
+		TRACE(2, ("xmt: %lx.%6.6u %s %s\n", (u_long)tv_xmt.tv_sec,
+			  (u_int)tv_xmt.tv_usec, dctx->name, stoa(dst)));
+	} else {
+		dec_pending_ntp(dctx->name, dst);
+	}
 
 	return;
 }
 
 
 /*
- * timeout_queries()  -- give up on unrequited NTP queries
+ * timeout_queries() -- give up on unrequited NTP queries
  */
 void
 timeout_queries(void)
@@ -701,21 +701,51 @@ timeout_queries(void)
 	sent_pkt *	spkt;
 	sent_pkt *	spkt_next;
 	long		age;
+	int didsomething = 0;
+
+	TRACE(3, ("timeout_queries: called to check %u items\n",
+		  (unsigned)COUNTOF(fam_listheads)));
 
 	gettimeofday_cached(base, &start_cb);
 	for (idx = 0; idx < COUNTOF(fam_listheads); idx++) {
 		head = fam_listheads[idx];
 		for (spkt = head; spkt != NULL; spkt = spkt_next) {
+			char xcst;
+
+			didsomething = 1;
+			switch (spkt->dctx->flags & CTX_xCST) {
+			    case CTX_BCST:
+				xcst = 'B';
+				break;
+
+			    case CTX_UCST:
+				xcst = 'U';
+				break;
+
+			    default:
+				INSIST(!"spkt->dctx->flags neither UCST nor BCST");
+				break;
+			}
+
 			spkt_next = spkt->link;
 			if (0 == spkt->stime || spkt->done)
 				continue;
 			age = start_cb.tv_sec - spkt->stime;
-			TRACE(3, ("%s %s age %ld\n", stoa(&spkt->addr),
-				  spkt->dctx->name, age));
-			if (age > ucst_timeout)
+			TRACE(3, ("%s %s %cCST age %ld\n",
+				  stoa(&spkt->addr),
+				  spkt->dctx->name, xcst, age));
+			if (age > response_timeout)
 				timeout_query(spkt);
 		}
 	}
+	// Do we care about didsomething?
+	TRACE(3, ("timeout_queries: didsomething is %d, age is %ld\n",
+		  didsomething, (long) (start_cb.tv_sec - start_tv.tv_sec)));
+	if (start_cb.tv_sec - start_tv.tv_sec > response_timeout) {
+		TRACE(3, ("timeout_queries: bail!\n"));
+		event_base_loopexit(base, NULL);
+		shutting_down = TRUE;
+	}
 }
 
 
@@ -729,8 +759,8 @@ void dec_pending_ntp(
 		check_exit_conditions();
 	} else {
 		INSIST(0 == n_pending_ntp);
-		TRACE(1, ("n_pending_ntp reached zero before dec for %s %s\n",
-			  name, stoa(server)));
+		TRACE(1, ("n_pending_ntp was zero before decrement for %s\n",
+			  hostnameaddr(name, server)));
 	}
 }
 
@@ -740,12 +770,29 @@ void timeout_query(
 	)
 {
 	sockaddr_u *	server;
+	char		xcst;
 
+
+	switch (spkt->dctx->flags & CTX_xCST) {
+	    case CTX_BCST:
+		xcst = 'B';
+		break;
+
+	    case CTX_UCST:
+		xcst = 'U';
+		break;
+
+	    default:
+		INSIST(!"spkt->dctx->flags neither UCST nor BCST");
+		break;
+	}
 	spkt->done = TRUE;
 	server = &spkt->addr;
-	msyslog(LOG_NOTICE, "%s %s no response after %d seconds",
-		spkt->dctx->name, stoa(server), ucst_timeout);
+	msyslog(LOG_INFO, "%s no %cCST response after %d seconds",
+		hostnameaddr(spkt->dctx->name, server), xcst,
+		response_timeout);
 	dec_pending_ntp(spkt->dctx->name, server);
+	return;
 }
 
 
@@ -802,7 +849,8 @@ sock_cb(
 	int		rc;
 
 	INSIST(sock4 == fd || sock6 == fd);
-	TRACE(3, ("sock_cb: event on sock%s:%s%s%s%s [UCST]\n",
+
+	TRACE(3, ("sock_cb: event on sock%s:%s%s%s%s\n",
 		  (fd == sock6)
 		      ? "6"
 		      : "4",
@@ -819,7 +867,7 @@ sock_cb(
 	}
 
 	/* Read in the packet */
-	rpktl = recvdata(fd, &sender, &r_pkt, sizeof(rbuf));
+	rpktl = recvdata(fd, &sender, &rbuf, sizeof(rbuf));
 	if (rpktl < 0) {
 		msyslog(LOG_DEBUG, "recvfrom error %m");
 		return;
@@ -1104,15 +1152,19 @@ handle_pkt(
 	const char *	hostname
 	)
 {
+	char		disptxt[32];
 	const char *	addrtxt;
 	struct timeval	tv_dst;
+	int		cnt;
 	int		sw_case;
 	int		digits;
+	int		stratum;
 	char *		ref;
 	char *		ts_str;
+	char *		leaptxt;
 	double		offset;
 	double		precision;
-	double		root_dispersion;
+	double		synch_distance;
 	char *		p_SNTP_PRETEND_TIME;
 	time_t		pretend_time;
 #if SIZEOF_TIME_T == 8
@@ -1181,7 +1233,7 @@ handle_pkt(
 		}
 
 		offset_calculation(rpkt, rpktl, &tv_dst, &offset,
-				   &precision, &root_dispersion);
+				   &precision, &synch_distance);
 		time_derived = TRUE;
 
 		for (digits = 0; (precision *= 10.) < 1.; ++digits)
@@ -1190,13 +1242,46 @@ handle_pkt(
 			digits = 6;
 
 		ts_str = tv_to_str(&tv_dst);
-		printf("%s %+.*f", ts_str, digits, offset);
-		if (root_dispersion > 0.)
-			printf(" +/- %f ", root_dispersion);
-		printf(" %s %s%s\n", hostname, stoa(host),
-		       (time_adjusted) 
-			   ? " [excess]"
-			   : "");
+		stratum = rpkt->stratum;
+		if (0 == stratum)
+				stratum = 16;
+
+		if (synch_distance > 0.) {
+			cnt = snprintf(disptxt, sizeof(disptxt),
+				       " +/- %f", synch_distance);
+			if ((size_t)cnt >= sizeof(disptxt))
+				snprintf(disptxt, sizeof(disptxt),
+					 "ERROR %d >= %d", cnt,
+					 (int)sizeof(disptxt));
+		} else {
+			disptxt[0] = '\0';
+		}
+
+		switch (PKT_LEAP(rpkt->li_vn_mode)) {
+		    case LEAP_NOWARNING:
+		    	leaptxt = "no-leap";
+			break;
+		    case LEAP_ADDSECOND:
+		    	leaptxt = "add-leap";
+			break;
+		    case LEAP_DELSECOND:
+		    	leaptxt = "del-leap";
+			break;
+		    case LEAP_NOTINSYNC:
+		    	leaptxt = "unsync";
+			break;
+		    default:
+		    	leaptxt = "LEAP-ERROR";
+			break;
+		}
+
+		msyslog(LOG_INFO, "%s %+.*f%s %s s%d %s%s", ts_str,
+			digits, offset, disptxt,
+			hostnameaddr(hostname, host), stratum,
+			leaptxt,
+			(time_adjusted)
+			    ? " [excess]"
+			    : "");
 		free(ts_str);
 
 		if (p_SNTP_PRETEND_TIME)
@@ -1220,7 +1305,7 @@ offset_calculation(
 	struct timeval *tv_dst,
 	double *offset,
 	double *precision,
-	double *root_dispersion
+	double *synch_distance
 	)
 {
 	l_fp p_rec, p_xmt, p_ref, p_org, tmp, dst;
@@ -1236,14 +1321,60 @@ offset_calculation(
 	NTOHL_FP(&rpkt->xmt, &p_xmt);
 
 	*precision = LOGTOD(rpkt->precision);
-	TRACE(3, ("offset_calculation: precision: %f\n", *precision));
 
-	*root_dispersion = FPTOD(p_rdsp);
+	TRACE(3, ("offset_calculation: LOGTOD(rpkt->precision): %f\n", *precision));
+
+	/* Compute offset etc. */
+	tmp = p_rec;
+	L_SUB(&tmp, &p_org);
+	LFPTOD(&tmp, t21);
+	TVTOTS(tv_dst, &dst);
+	dst.l_ui += JAN_1970;
+	tmp = p_xmt;
+	L_SUB(&tmp, &dst);
+	LFPTOD(&tmp, t34);
+	*offset = (t21 + t34) / 2.;
+	delta = t21 - t34;
+
+	// synch_distance is:
+	// (peer->delay + peer->rootdelay) / 2 + peer->disp
+	// + peer->rootdisp + clock_phi * (current_time - peer->update)
+	// + peer->jitter;
+	//
+	// and peer->delay = fabs(peer->offset - p_offset) * 2;
+	// and peer->offset needs history, so we're left with
+	// p_offset = (t21 + t34) / 2.;
+	// peer->disp = 0; (we have no history to augment this)
+	// clock_phi = 15e-6; 
+	// peer->jitter = LOGTOD(sys_precision); (we have no history to augment this)
+	// and ntp_proto.c:set_sys_tick_precision() should get us sys_precision.
+	//
+	// so our answer seems to be:
+	//
+	// (fabs(t21 + t34) + peer->rootdelay) / 3.
+	// + 0 (peer->disp)
+	// + peer->rootdisp
+	// + 15e-6 (clock_phi)
+	// + LOGTOD(sys_precision)
+
+	INSIST( FPTOD(p_rdly) >= 0. );
+#if 1
+	*synch_distance = (fabs(t21 + t34) + FPTOD(p_rdly)) / 3.
+		+ 0.
+		+ FPTOD(p_rdsp)
+		+ 15e-6
+		+ 0.	/* LOGTOD(sys_precision) when we can get it */
+		;
+	INSIST( *synch_distance >= 0. );
+#else
+	*synch_distance = (FPTOD(p_rdly) + FPTOD(p_rdsp))/2.0;
+#endif
 
 #ifdef DEBUG
-	if (debug > 2) {
+	if (debug > 3) {
 		printf("sntp rootdelay: %f\n", FPTOD(p_rdly));
-		printf("sntp rootdisp: %f\n", *root_dispersion);
+		printf("sntp rootdisp: %f\n", FPTOD(p_rdsp));
+		printf("sntp syncdist: %f\n", *synch_distance);
 
 		pkt_output(rpkt, rpktl, stdout);
 
@@ -1258,21 +1389,11 @@ offset_calculation(
 	}
 #endif
 
-	/* Compute offset etc. */
-	tmp = p_rec;
-	L_SUB(&tmp, &p_org);
-	LFPTOD(&tmp, t21);
-	TVTOTS(tv_dst, &dst);
-	dst.l_ui += JAN_1970;
-	tmp = p_xmt;
-	L_SUB(&tmp, &dst);
-	LFPTOD(&tmp, t34);
-	*offset = (t21 + t34) / 2.;
-	delta = t21 - t34;
-
 	TRACE(3, ("sntp offset_calculation:\trec - org t21: %.6f\n"
 		  "\txmt - dst t34: %.6f\tdelta: %.6f\toffset: %.6f\n",
 		  t21, t34, delta, *offset));
+
+	return;
 }
 
 
@@ -1291,7 +1412,7 @@ set_li_vn_mode (
 		leap = 3;
 	}
 
-	if (version < 0 || version > 7) {
+	if ((unsigned char)version > 7) {
 		msyslog(LOG_DEBUG, "set_li_vn_mode: version < 0 or > 7, using 4");
 		version = 4;
 	}
@@ -1441,10 +1562,10 @@ gettimeofday_cached(
 				progname);
 			exit(1);
 		}
-		timeval_sub(&diff, &systemt, &latest);
+		diff = sub_tval(systemt, latest);
 		if (debug > 1)
 			printf("system minus cached %+ld.%06ld\n",
-			       (long)diff.tv_sec, diff.tv_usec);
+			       (long)diff.tv_sec, (long)diff.tv_usec);
 		if (0 != cgt_rc || labs((long)diff.tv_sec) < 3600) {
 			/*
 			 * Either use_monotonic == 0, or this libevent
@@ -1453,21 +1574,22 @@ gettimeofday_cached(
 		} else {
 			mono.tv_sec = ts.tv_sec;
 			mono.tv_usec = ts.tv_nsec / 1000;
-			timeval_sub(&diff, &latest, &mono);
+			diff = sub_tval(latest, mono);
 			if (debug > 1)
 				printf("cached minus monotonic %+ld.%06ld\n",
-				       (long)diff.tv_sec, diff.tv_usec);
+				       (long)diff.tv_sec, (long)diff.tv_usec);
 			if (labs((long)diff.tv_sec) < 3600) {
 				/* older libevent2 using monotonic */
-				timeval_sub(&offset, &systemt, &mono);
-				msyslog(LOG_NOTICE,
-					"Offsetting CLOCK_MONOTONIC times by %+ld.%06ld\n",
-					(long)offset.tv_sec, offset.tv_usec);
+				offset = sub_tval(systemt, mono);
+				TRACE(1, ("%s: Offsetting libevent CLOCK_MONOTONIC times  by %+ld.%06ld\n",
+					 "gettimeofday_cached",
+					 (long)offset.tv_sec,
+					 (long)offset.tv_usec));
 			}
 		}
 		offset_ready = TRUE;
 	}
-	timeval_add(&adj_cached, &cached, &offset);
+	adj_cached = add_tval(cached, offset);
 	*caller_tv = adj_cached;
 
 	return 0;
